import { AfterContentInit, AfterViewChecked, AfterViewInit, Component, ElementRef, Input, OnChanges, OnDestroy, ViewChild } from '@angular/core';
import { Message } from '../../../models/message';
import { ChatService } from '../../../services/chat.service';
import { Channel } from '../../../models/channel';
import { Subscription } from 'rxjs';

@Component({
  selector: 'app-feed',
  standalone: false,
  templateUrl: './feed.component.html',
  styleUrl: './feed.component.scss'
})
export class FeedComponent implements OnChanges, OnDestroy, AfterViewInit {
  private readonly DEFAULT_CHANNEL_ID: number = 1;

  @Input('channel') public channel?: Channel;
  public messages: Message[] = [];

  public subscription?: Subscription;

  constructor(private chatService: ChatService, private element: ElementRef<HTMLElement>) { }

  ngAfterViewInit(): void {
    this.scrollToBottom();
  }

  ngOnChanges(): void {
    if (this.subscription) {
      this.subscription.unsubscribe();
      this.messages = [];
    }

    this.subscription =
      this.chatService.GetMessages(this.channel?.id ?? this.DEFAULT_CHANNEL_ID)
        .subscribe(message => {
          this.messages.push(message);

          if (this.isAtBottom()) this.scrollToBottom();
        });
  }

  ngOnDestroy(): void {
    if (this.subscription) {
      this.subscription.unsubscribe();
    }
  }

  private isAtBottom(): boolean {
    let element = this.element.nativeElement;
    return Math.abs(element.scrollHeight - element.clientHeight - element.scrollTop) <= 100;
  }

  private scrollToBottom(): void {
    setTimeout(() => this.element.nativeElement.scrollTop = this.element.nativeElement.scrollHeight, 100);
  }
}
